/*
 * ====================================================================
 * 
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 

package org.apache.jasper.compiler;

import java.io.OutputStream;
import java.io.IOException;
import java.io.Writer;

import org.apache.jasper.Constants;

/**
 * Used to escape unicode characters with \ u's. 
 *
 * @author Anselm Baird-Smith
 */
public class EscapeUnicodeWriter extends Writer {
    private OutputStream out     = null;
    private byte         bytes[] = new byte[6];
    
    public EscapeUnicodeWriter(OutputStream out) {
        this.out = out;
    }
    
    public void write(char buf[], int off, int len) 
        throws IOException
    {
        if (out == null)
            throw new IOException(Constants.getString("jsp.error.stream.closed"));
        
        int ci = off, end = off + len;
        while ( --len >= 0 ) {
            int ch = buf[off++] & 0xffff;
            /*
             * Write out unicode characters as \u0000
             */
            if ((ch < 0x20 || ch > 0x7e) && 
                (ch != '\n' && ch != '\r' && ch != '\t')) {
                bytes[0] = (byte) '\\';
                bytes[1] = (byte) 'u';
                bytes[2] = (byte) Character.forDigit((ch & 0xf000) >> 12, 16);
                bytes[3] = (byte) Character.forDigit((ch & 0x0f00) >>  8, 16);
                bytes[4] = (byte) Character.forDigit((ch & 0x00f0) >>  4, 16);
                bytes[5] = (byte) Character.forDigit((ch & 0x000f) >>  0, 16);
                out.write(bytes);
            } else {
                out.write((byte) ch);
            }
        }
    }
    
    public void flush() throws IOException {
        out.flush();
    }
    
    public void close() 
        throws IOException
    {
        if (out == null) 
            return;
        out.close();
        out = null;
    }
}
